// T15simState.sce
// SIMULATION WITH RM IN A STATE-SPACE FORM
// Experiments
// - extend to the 3-rd order model
//   y(t)=b0.u(t)+a1.y(t-1)+b1.u(t-1)+a2.y(t-2)+b2.u(t-2)+
//        +a3.y(t-3)+b3.u(t-3)+k+e(t)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1)
rand('seed',0)

// PARAMETERS OF THE SIMULATION
nd=100;                       // number of data

et=rand(1,nd,'n');
ut=rand(1,nd,'n');            // input
a=[.6 .1]; b0=.8; b=[.3 .2]; k=2; cv=.1; // model parameterers

// REGRESSION REALIZATION
yr(1)=0; yr(2)=1;
for t=3:nd
  er=sqrt(cv)*et(t);
  yr(t)=a*[yr(t-1) yr(t-2)]'+b0*ut(t)+b*[ut(t-1) ut(t-2)]'+k+er;
end

// STATE-SPACE REALIZATION
M=[a(1) b(1) a(2) b(2) k
   0    0    0    0    0
   1    0    0    0    0
   0    1    0    0    0
   0    0    0    0    1];
N=[b0 1 zeros(1,3)]'
A=[1 zeros(1,4)];
B=0; 

yt(1)=0; yt(2)=1;
xt(:,2)=[yt(2) ut(2) yt(1) ut(1) 1]';  // initial conditions for state

// time loop of simulation
for t=3:nd
  es=[sqrt(cv)*et(t) zeros(1,4)]';
  xt(:,t)=M*xt(:,t-1)+N*ut(t)+es;
  yt(t)=A*xt(:,t);
end

// RESULTS OF SIMULATION
scf(1); plot(1:nd,yt,1:nd,yr,'.','markersize',4)
legend('state model','regression model');
