// T22estCont_B2.sce
// ESTIMATION OF 2ND ORDER REGRESSION MODEL
// The model for simulation differs from that for estimation
// (simulation model: 3rd order; estimation: 1st order)
// i.e. MODEL STRUCTURE MISMATCH is tackled.
// - Bayesian on-line estimation with statistic update
// Experiments 
// - try various input signals - set inp = 1,2,3,4
// - set different noise variance r=0,.1,1,10
// - define other regression coefficients a,b,k
// - you can also change the orders of models for simulation 
//   as well as for estimation (adjust the beginning of loop)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1)

// SIMULATION
// parameters
nd=100;                       // length of data
a=[.1 -.8 .3];                // parameters at yt
b=[1 .2 -.5 -.8];             // parameters at ut
k=0;                          // constant (model absolute term)
s=.01;                        // noise variance
inp=1;                        // selection of input

yt(1)=0; yt(2)=0; yt(3)=0;    // initial conditions for output
                                             // inputs at disposal 
select inp
case 1, ut=ones(nd,1);                       // one jump
case 2, ut=sin(10*%pi*(1:nd)/nd)';           // several jumps
case 3, ut=sign(10*sin(10*%pi*(1:nd)/nd))';  // sine wave
case 4, ut=.1*rand(nd,1,'n');                // white noise
end

// time loop
th=[a b k]';                  // vector of parameters
for t=4:nd
  // regression vector
  ps=[yt(t-1) yt(t-2) yt(t-3) ut(t) ut(t-1) ut(t-2) ut(t-3) 1]';
  // regression model
  yt(t)=th'*ps+s*rand(1,1,'norm'); 
end

// ESTIMATION
V=1e-8*eye(5,5);              // initial information matrix
for t=3:nd
  psi=[yt(t:-1:t-1); ut(t:-1:t-1); 1];       // reg. vector
  V=V+psi*psi';                              // updt of information matrix
  Vy=V(1,1); Vyp=V(2:$,1); Vp=V(2:$,2:$);    // divisioning of inf. matrix
  thE(:,t)=inv(Vp+1e-8*eye(Vp))*Vyp;         // pt estimates of reg. coefficints
  r(t)=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/t; // pt estimates of noise variance
end

// Results
set(scf(1),'position',[500 60 600 500])
for i=1:4
  subplot(6,1,i),plot(thE(i,:))
  if i==1, title('Regression coefficients'), end
end
set(scf(2),'position',[50 60 400 200])
plot(r)
title('Noise variance')
set(scf(3),'position',[50 360 400 200])
plot(1:nd,yt,1:nd,ut)
legend('output','input');
title 'Dataset for estimation'

disp(th','Simulated parameters')
disp(thE(:,$)','Easimated parameters')
