// T32preCont_Adapt.sce
// N-STEP PREDICTION WITH CONTINUOUS MODEL (WITH ESTIMATION)
// Experiments
// Change: - np = number of steps of prediction
//         - r  = noise variance (effect on estimation)
//         - th = model parametrs
//         - u  = input signal (effect on estimation)
// -------------------------------------------------------------------
exec("ScIntro.sce",-1),mode(0)

nd=100;                            // number of data
np=5;                              // length of prediction (np>=1)
nz=3;                              // starting time (ord+1)
//  b0 a1 b1  a2 b2 k
th=[1 .4 -.3 -.5 .1 1]';           // regression coefficients
r=.2;                              // noise variance
u=sin(4*%pi*(1:nd)/nd)+0*rand(1,nd,'norm'); // input
y(1)=1; y(2)=-1;                   // prior data
Eth=rand(6,1,'n');                 // prior parametrs (random)

for t=nz:(nd-np)                   // time loop (on-line tasks)
  // prediction
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]'; // regression vector
  yy=ps'*Eth;                           // first prediction at t+1
  for j=1:np                            // loop of predictions for t+2,..,t+np
    tj=t+j;                             // future times for prediction
    ps=[u(tj); yy; ps(1:$-3); 1];       // reg.vecs with predicted outputs
    yy=ps'*Eth;                         // new prediction (partial)
  end
  yp(t+np)=yy;                            // final prediction for time t+np

  // simulation
  ps=[u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]';  // regression vector for sim.
  y(t)=ps'*th+sqrt(r)*rand(1,1,'norm');      // output generation
 
  // estimation
  Ps=[y(t) u(t) y(t-1) u(t-1) y(t-2) u(t-2) 1]';  // reg.vect. for estim.
  if t==nz, V=1e-8*eye(length(Ps)); end // initial information matrix
  V=V+Ps*Ps';                           // update of statistics
  Vp=V(2:$,2:$);
  Vyp=V(2:$,1);
  Eth=inv(Vp+1e-8*eye(Vp))*Vyp;         // point estimates
  Et(:,t)=Eth(:,1);                     // stor est. parameters
end

// Results
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)

set(scf(1),'position',[300 100 1200 400]);
subplot(121),plot(Et')
set(gca(),"data_bounds",[0 nd+1 -1 2])
title('Evolution of estimated parameters')
subplot(122)
s=(np+3):(nd-np);
plot(s,y(s),'.:',s,yp(s),'rx')
set(gca(),"data_bounds",[1 nd -3 5])
legend('output','prediction');
title([string(np),'-steps ahead prediction'])

SE1=rpe(y(np+(1:10)),yp(np+(1:10)))
SE2=rpe(y(nd-np+(-20:-10)),yp(nd-np+(-20:-10)))
