// T32preCont_Adapt3.sce
// np-STEP PREDICTION WITH CONTINUOUS MODEL (WITh ESTIMATION)
// REAL DATA (intensity) from Strahov tunnel are used
// i.e. MODEL STRUCTURE MISMATCH is tackled.
// Experiments
// Change: - np  = number of steps of prediction
//         - ord = order of the model for estimation 
// Result: - visual comparison of yt and yp
//         - RPE = relative prediction error
// -------------------------------------------------------------------
exec("ScIntro.sce",-1),mode(0)

np=5;                              // length of prediction (np>=1)
ord=2;                             // order of the estimated model
// data selection
k=3;                               // which day is selected
nz=50;                             // beginning of the day
nd=288*2;                          // length of the whole one day data

// DATA
dtAll=csvRead('STRAHOV.csv',';');  
dt=dtAll((k-1)*288+(nz+1:nd),1);  
nth=ord+2;                         // size of V
V=1e-8*eye(nth,nth);               // initial information matrix
thE=rand(nth-1,1,'n');             // prior parametrs
yt=dt(1:ord);                      // prior data

for t=ord+1:(nd-np-nz)             // time loop (on-line tasks)
//  PREDICTION
  ps=[yt(t-1:-1:t-ord); 1];        // regression vector
  yy=ps'*thE;                      // first prediction at t+1
  for j=1:np                       // loop of predictions for t+2,...,t+np
    tj=t+j;                        // future times for prediction
    ps=[yy; ps(1:(ord-1)); 1];     // reg. vector
    yy=ps'*thE;                    // new prediction (partial)
  end
  yp(t+np,1)=yy;                   // final prediction for time t+np

  // DATA MEASUREMENT (as if)
  yt(t)=dt(t);                     // measuring of output

  // ESTIMATION
  psi=[yt(t:-1:t-ord); 1];                   // reg. vector
  V=V+psi*psi';                              // updt of information matrix
  Vy=V(1,1); Vyp=V(2:$,1); Vp=V(2:$,2:$);    // divisioning of inf. matrix
  thE(:,1)=inv(Vp+1e-8*eye(Vp))*Vyp;         // pt estimates of reg. coefficints
  r(t)=(Vy-Vyp'*inv(Vp+1e-8*eye(Vp))*Vyp)/t; // pt estimates of noise variance
  Et(:,t)=thE;                               // stor est. parameters
end

// Results
// evolution of parametrs
set(scf(1),'position',[100 300 1200 400]);
subplot(121),plot(Et')
set(gca(),"data_bounds",[0 nd+1 -1 5])
title('Evolution of estimated parameters')
legend('a1E','a2E','kE');
// comparison of yt and yp
subplot(122)
s=1:length(yt);
plot(s,yt(s),'.:',s,yp(s),'rx')
set(gca(),"data_bounds",[1 nd -4 max(yt)+5])
legend('output','prediction');
title([string(np),'- step ahead prediction'])

s=101:length(yt);
RPE=rpe(yt(s),yp(s))
