// T46statEst_KF.sce
// STATE ESTIMATION (KALMAN FILTER) 
// Experiments
// - change model parameters M,N,A,B
// - set different system and model covariances rw,rv and Rw,Rv
// - try lower stat-estimate covariance Rx
// -------------------------------------------------------------------
exec("ScIntro.sce",-1), getd()

nd=200;                // number of data
// SIMULATION
// parameters of simulation
M=[.8 .1
   .3 .6];
N=[.5 -.5]';
A=[.9 -.2];
B=0;   
rw=.1*eye(2,2);
rv=.1;
xt(:,1)=[0 0]';
ut=rand(1:nd,'n');
// time loop of simulation
for t=2:nd
  xt(:,t)=M*xt(:,t-1)+N*ut(t)+rw*rand(2,1,'n');
  yt(t)  =A*xt(:,t)+B*ut(t)+rv*rand(1,1,'n');
end  
  
// ESTIMATION 
// initialization of estimation
Rw=.1*eye(2,2);                     // state noise covariance
Rv=.1;                             // output noise vovariance
Rx=1000*eye(2,2);                  // estimated state covariance
xp(:,1)=zeros(2,1);                // initial state
// loop for state estimation
for t=2:nd
  [xp(:,t),Rx,yp(t)]=Kalman(xp(:,t-1),yt(t),ut(t),M,N,[],A,B,[],Rw,Rv,Rx);
end

// RESULTS
subplot(311),plot(1:nd,xt(1,:),1:nd,xp(1,:))
set(gcf(),"position",[700 100 600 500])
title('First state entry')
legend('state','estimate')
subplot(312),plot(1:nd,xt(2,:),1:nd,xp(2,:))
title('Second state entry')
legend('state','estimate')
subplot(313),plot(1:nd,yt,1:nd,yp')
title('Output')
legend('output','estimate')
