function [f,th]=filt3(x,vL,nv,dL,nd,p2,I,ni)
  // f=filt3(x,vL,nv,dL,nd,p2,I,ni)
  // approximation by the 3rd order polynomial
  // x   signal to be approximated
  // vL  value on the left (border condition)
  // nv  number of data for fixing the left border
  // dL  derivative on the left (border condition)
  // nd  number of data for fixing the equality of derivatives
  // p2  relative number of data for fixing zero second deriv.
  // I   integral of the approximative polynomial
  // ni  number of data for fixing the pre-scribed integral
  //
  if argn(2)<7, I=0; ni=0; end

  n=max(size(x));                       // data length
  d=[];
  for i=(1:n)-1                         // data generation
    d=[d [x(i+1); 1; i; i^2; i^3]];     // fitting data
    d=[d [0; 0; 0; 2; 6*i]*p2];         // 2nd derivative cond.
  end
  d=[d [vL; 1; 0; 0; 0]*nv];            // cont. condidion
  d=[d [dL; 0; 1; 0; 0]*nd];            // derivatives on left
  d=[d [I; n-1; (n-1)^2/2; (n-1)^3/3; i^4/4]*ni]; // integral
  V=d*d';                               // information matrix
  V=V+eye(V)*1e-5;                      // regularization
  th=v2thN(V/n,1);                      // point estimates
  f=zeros(1,n);
  for i=(1:n)-1                         // polynomial generation
    f(i+1)=th'*[1; i; i^2; i^3];
  end
endfunction
